library getx_tr_maker;

import 'dart:convert';
import 'dart:io';
import 'package:path/path.dart' as path;
import 'package:translator/translator.dart';

/// A Calculator.
class Calculator {
  /// Returns [value] plus 1.
  int addOne(int value) => value + 1;
}

main(List<String> args) async {
  if (args.length == 1 && (args[0] == '--help' || args[0] == '-h') ||
      args.length < 3) {
    print('flutter pub run getx_tr_maker jsonfile from [to1, to2]');
    print(
        'ex : flutter pub run getx_tr_maker i18n/ko_KR.json ko-KR "en-US,de-DE,gl-ES,gu-IN,el-GR,nl-NL,da-DK,lv-LV,ru-RU,ro-RO,lt-LT,mr-IN,mk-MK,ms-BN,ms-MY,mn-MN,eu-ES,vi-VN​,be-BY,bg-BG,sr-Latn-CS,sw-KE,sv-SE,es-ES,sk-SK,sl-SI,ar-DZ,hy-AM,is-IS,ja-JP,zh-CN"');
    return 3;
  }
  var root = await findProjectRoot();
  var path = "$root/${args[0]}";
  File jsonFile = File(path);
  Map<String, dynamic> json = jsonDecode(jsonFile.readAsStringSync());
  var listTo = args[2].split(",");
  var from = args[1];

  final translator = GoogleTranslator();
  Map<String, Map<String, String>> keys = {};

  await Future.forEach(json.keys, (String k) async {
    var v = json[k];
    for (var to in listTo) {
      to = to.trim();
      var tmpTo = to.toLowerCase();
      if (!_langs.containsKey(tmpTo)) {
        tmpTo = tmpTo.split('-').first;
      }
      try {
        var translation = await translator.translate(v,
            from: from.split('-').first, to: tmpTo);
        if (keys[from] == null) {
          keys[from] = {};
        }
        keys[from]![k] = v;

        if (keys[to] == null) {
          keys[to] = {};
        }

        keys[to]![k] = translation.text;
      } catch (e) {
        print("${e} : to ${to}");
      }
    }
  });

  var dartContents = makeDartContents(keys);
  var dartFilepath = "$root/lib/i18n.dart";
  File dartFile = File(dartFilepath);
  dartFile.create(recursive: true);
  dartFile.writeAsStringSync(dartContents);
  print("success! ${dartFilepath}");
}

makeDartContents(Map<String, Map<String, String>> keys) {
  var data = """
  import 'package:get/get.dart';
  class Messages extends Translations {
    @override
    Map<String, Map<String, String>> get keys => ${json.encode(keys)};
  }
  """;
  return data;
}

Future<String> findProjectRoot({Directory? from}) async {
  from = from ?? Directory.current;
  if (await File(path.join(from.path, 'pubspec.yaml')).exists()) {
    return from.path;
  }
  return findProjectRoot(from: from.parent);
}

final _langs = {
  'auto': 'Automatic',
  'af': 'Afrikaans',
  'sq': 'Albanian',
  'am': 'Amharic',
  'ar': 'Arabic',
  'hy': 'Armenian',
  'az': 'Azerbaijani',
  'eu': 'Basque',
  'be': 'Belarusian',
  'bn': 'Bengali',
  'bs': 'Bosnian',
  'bg': 'Bulgarian',
  'ca': 'Catalan',
  'ceb': 'Cebuano',
  'ny': 'Chichewa',
  'zh-cn': 'Chinese Simplified',
  'zh-tw': 'Chinese Traditional',
  'co': 'Corsican',
  'hr': 'Croatian',
  'cs': 'Czech',
  'da': 'Danish',
  'nl': 'Dutch',
  'en': 'English',
  'eo': 'Esperanto',
  'et': 'Estonian',
  'tl': 'Filipino',
  'fi': 'Finnish',
  'fr': 'French',
  'fy': 'Frisian',
  'gl': 'Galician',
  'ka': 'Georgian',
  'de': 'German',
  'el': 'Greek',
  'gu': 'Gujarati',
  'ht': 'Haitian Creole',
  'ha': 'Hausa',
  'haw': 'Hawaiian',
  'iw': 'Hebrew',
  'hi': 'Hindi',
  'hmn': 'Hmong',
  'hu': 'Hungarian',
  'is': 'Icelandic',
  'ig': 'Igbo',
  'id': 'Indonesian',
  'ga': 'Irish',
  'it': 'Italian',
  'ja': 'Japanese',
  'jw': 'Javanese',
  'kn': 'Kannada',
  'kk': 'Kazakh',
  'km': 'Khmer',
  'ko': 'Korean',
  'ku': 'Kurdish (Kurmanji)',
  'ky': 'Kyrgyz',
  'lo': 'Lao',
  'la': 'Latin',
  'lv': 'Latvian',
  'lt': 'Lithuanian',
  'lb': 'Luxembourgish',
  'mk': 'Macedonian',
  'mg': 'Malagasy',
  'ms': 'Malay',
  'ml': 'Malayalam',
  'mt': 'Maltese',
  'mi': 'Maori',
  'mr': 'Marathi',
  'mn': 'Mongolian',
  'my': 'Myanmar (Burmese)',
  'ne': 'Nepali',
  'no': 'Norwegian',
  'ps': 'Pashto',
  'fa': 'Persian',
  'pl': 'Polish',
  'pt': 'Portuguese',
  'pa': 'Punjabi',
  'ro': 'Romanian',
  'ru': 'Russian',
  'sm': 'Samoan',
  'gd': 'Scots Gaelic',
  'sr': 'Serbian',
  'st': 'Sesotho',
  'sn': 'Shona',
  'sd': 'Sindhi',
  'si': 'Sinhala',
  'sk': 'Slovak',
  'sl': 'Slovenian',
  'so': 'Somali',
  'es': 'Spanish',
  'su': 'Sundanese',
  'sw': 'Swahili',
  'sv': 'Swedish',
  'tg': 'Tajik',
  'ta': 'Tamil',
  'te': 'Telugu',
  'th': 'Thai',
  'tr': 'Turkish',
  'uk': 'Ukrainian',
  'ur': 'Urdu',
  'uz': 'Uzbek',
  'ug': 'Uyghur',
  'vi': 'Vietnamese',
  'cy': 'Welsh',
  'xh': 'Xhosa',
  'yi': 'Yiddish',
  'yo': 'Yoruba',
  'zu': 'Zulu'
};

    // "menu" : "메뉴",
    // "hello" : "Hello World"